@@ -4,8 +4,8 @@
 
 ;; Author: Milan Glacier <dev@milanglacier.com>
 ;; Maintainer: Milan Glacier <dev@milanglacier.com>
-;; Package-Version: 20251218.2234
-;; Package-Revision: d3ce06dfd33f
+;; Package-Version: 20260203.254
+;; Package-Revision: dd8301f05dce
 ;; URL: https://github.com/milanglacier/minuet-ai.el
 ;; Package-Requires: ((emacs "29") (plz "0.9") (dash "2.19.1"))
 
@@ -336,6 +336,7 @@ const processedData = transformData(rawData, {
      :language-and-tab minuet--default-chat-input-language-and-tab-function
      :context-before-cursor minuet--default-chat-input-before-cursor-function
      :context-after-cursor minuet--default-chat-input-after-cursor-function)
+    :transform ()
     :optional nil)
   "Config options for Minuet Claude provider.")
 
@@ -354,6 +355,7 @@ const processedData = transformData(rawData, {
      :language-and-tab minuet--default-chat-input-language-and-tab-function
      :context-before-cursor minuet--default-chat-input-before-cursor-function
      :context-after-cursor minuet--default-chat-input-after-cursor-function)
+    :transform ()
     :optional nil)
   "Config options for Minuet OpenAI provider.")
 
@@ -386,6 +388,7 @@ const processedData = transformData(rawData, {
      :language-and-tab minuet--default-chat-input-language-and-tab-function
      :context-before-cursor minuet--default-chat-input-before-cursor-function
      :context-after-cursor minuet--default-chat-input-after-cursor-function)
+    :transform ()
     :optional nil)
   "Config options for Minuet OpenAI compatible provider.")
 
@@ -418,6 +421,7 @@ const processedData = transformData(rawData, {
      :language-and-tab minuet--default-chat-input-language-and-tab-function
      :context-before-cursor minuet--default-chat-input-before-cursor-function
      :context-after-cursor minuet--default-chat-input-after-cursor-function)
+    :transform ()
     :optional nil)
   "Config options for Minuet Gemini provider.")
 
@@ -970,10 +974,9 @@ many lines.  Without a prefix argument, accept only the first line."
   "Get the api-key from API-KEY.
 API-KEY can be a string (as an environment variable) or a function.
 Return nil if not exists or is an empty string."
-  (let ((key (if (stringp api-key)
-                 (getenv api-key)
-               (when (functionp api-key)
-                 (funcall api-key)))))
+  (let ((key (cond
+              ((stringp api-key) (getenv api-key))
+              ((functionp api-key) (funcall api-key)))))
     (when (or (null key)
               (string-empty-p key))
       (minuet--log
@@ -1042,6 +1045,16 @@ If using ollama you can just set it to 'TERM'." api-key)
           ;; replace placeholders that are not replaced
           tmpl (replace-regexp-in-string "{{{.*}}}" "" tmpl))))
 
+(defun minuet--apply-request-transform (options end-point headers body)
+  "Apply OPTIONS transform functions to END-POINT, HEADERS, and BODY.
+Return a plist with :end-point, :headers, and :body."
+  (let ((transformed (list :end-point end-point
+                           :headers headers
+                           :body body)))
+    (dolist (fn (plist-get options :transform))
+      (setq transformed (or (funcall fn transformed) transformed)))
+    transformed))
+
 (defun minuet--openai-fim-complete-base (options get-text-fn context callback)
   "The base function to complete code with openai fim API.
 OPTIONS are the provider options.  GET-TEXT-FN are the function to get
@@ -1052,7 +1065,6 @@ arrive."
          ;; Initialize input components
          (name (plist-get options :name))
          (end-point (plist-get options :end-point))
-         (transform-functions (plist-get options :transform))
          (body `(,@(plist-get options :optional)
                  :stream t
                  :model ,(plist-get options :model)
@@ -1068,14 +1080,7 @@ arrive."
                     ("Accept" . "application/json")
                     ("Authorization" .
                      ,(concat "Bearer " (minuet--get-api-key (plist-get options :api-key))))))
-         ;; Apply transformations
-         (transformed `(:end-point ,end-point
-                        :headers ,headers
-                        :body ,body))
-         (transformed (progn (dolist (fn transform-functions)
-                               (setq transformed (or (funcall fn transformed) transformed)))
-                             transformed))
-         ;; Extract transformed components
+         (transformed (minuet--apply-request-transform options end-point headers body))
          (end-point (plist-get transformed :end-point))
          (headers (plist-get transformed :headers))
          (body (plist-get transformed :body))
@@ -1168,42 +1173,48 @@ and :content keys."
 OPTIONS are the provider options.  the completion items from json.
 CONTEXT is to be used to build the prompt.  CALLBACK is the function
 to be called when completion items arrive."
-  (minuet--with-temp-response
-    (push
-     (plz 'post (plist-get options :end-point)
-       :headers
-       `(("Content-Type" . "application/json")
-         ("Accept" . "application/json")
-         ("Authorization" . ,(concat "Bearer " (minuet--get-api-key (plist-get options :api-key)))))
-       :timeout minuet-request-timeout
-       :body
-       (json-serialize
-        `(,@(plist-get options :optional)
-          :stream t
-          :model ,(plist-get options :model)
-          :messages ,(vconcat
-                      `((:role "system"
-                         :content ,(minuet--make-system-prompt (plist-get options :system))))
-                      (minuet--eval-value (plist-get options :fewshots))
-                      (--> (minuet--make-chat-llm-shot context options)
-                           minuet--create-chat-messages-from-list))))
-       :as 'string
-       :filter (minuet--make-process-stream-filter --response--)
-       :then
-       (lambda (json)
-         (when-let* ((result (minuet--stream-decode json #'minuet--openai-get-text-fn))
-                     (completion-items (minuet--parse-completion-itmes-default result))
-                     (completion-items (minuet--filter-context-sequence-in-items
-                                        completion-items
-                                        context))
-                     (completion-items (minuet--remove-spaces completion-items)))
-           ;; insert the current result into the completion items list
-           (funcall callback completion-items)))
-       :else
-       (lambda (err)
-         (minuet--handle-chat-completion-timeout
-          context err --response-- #'minuet--openai-get-text-fn "OpenAI" callback)))
-     minuet--current-requests)))
+  (let* ((end-point (plist-get options :end-point))
+         (headers `(("Content-Type" . "application/json")
+                    ("Accept" . "application/json")
+                    ("Authorization" .
+                     ,(concat "Bearer " (minuet--get-api-key (plist-get options :api-key))))))
+         (body `(,@(plist-get options :optional)
+                 :stream t
+                 :model ,(plist-get options :model)
+                 :messages ,(vconcat
+                             `((:role "system"
+                                :content ,(minuet--make-system-prompt
+                                           (plist-get options :system))))
+                             (minuet--eval-value (plist-get options :fewshots))
+                             (--> (minuet--make-chat-llm-shot context options)
+                                  minuet--create-chat-messages-from-list))))
+         (transformed (minuet--apply-request-transform options end-point headers body))
+         (end-point (plist-get transformed :end-point))
+         (headers (plist-get transformed :headers))
+         (body-json (json-serialize (plist-get transformed :body))))
+    (minuet--with-temp-response
+      (push
+       (plz 'post end-point
+         :headers headers
+         :timeout minuet-request-timeout
+         :body body-json
+         :as 'string
+         :filter (minuet--make-process-stream-filter --response--)
+         :then
+         (lambda (json)
+           (when-let* ((result (minuet--stream-decode json #'minuet--openai-get-text-fn))
+                       (completion-items (minuet--parse-completion-itmes-default result))
+                       (completion-items (minuet--filter-context-sequence-in-items
+                                          completion-items
+                                          context))
+                       (completion-items (minuet--remove-spaces completion-items)))
+             ;; insert the current result into the completion items list
+             (funcall callback completion-items)))
+         :else
+         (lambda (err)
+           (minuet--handle-chat-completion-timeout
+            context err --response-- #'minuet--openai-get-text-fn "OpenAI" callback)))
+       minuet--current-requests))))
 
 (defun minuet--openai-complete (context callback)
   "Complete code with OpenAI.
@@ -1225,43 +1236,48 @@ CONTEXT and CALLBACK will be passed to the base function."
   "Complete code with Claude.
 CONTEXT is to be used to build the prompt.  CALLBACK is the function
 to be called when completion items arrive."
-  (minuet--with-temp-response
-    (push
-     (plz 'post (plist-get minuet-claude-options :end-point)
-       :headers `(("Content-Type" . "application/json")
-                  ("Accept" . "application/json")
-                  ("x-api-key" . ,(minuet--get-api-key (plist-get minuet-claude-options :api-key)))
-                  ("anthropic-version" . "2023-06-01"))
-       :timeout minuet-request-timeout
-       :body
-       (json-serialize
-        (let ((options minuet-claude-options))
-          `(,@(plist-get options :optional)
-            :stream t
-            :model ,(plist-get options :model)
-            :system ,(minuet--make-system-prompt (plist-get options :system))
-            :max_tokens ,(plist-get options :max_tokens)
-            :messages ,(vconcat
-                        (minuet--eval-value (plist-get options :fewshots))
-                        (--> (minuet--make-chat-llm-shot context options)
-                             minuet--create-chat-messages-from-list)))))
-       :as 'string
-       :filter (minuet--make-process-stream-filter --response--)
-       :then
-       (lambda (json)
-         (when-let* ((result (minuet--stream-decode json #'minuet--claude-get-text-fn))
-                     (completion-items (minuet--parse-completion-itmes-default result))
-                     (completion-items (minuet--filter-context-sequence-in-items
-                                        completion-items
-                                        context))
-                     (completion-items (minuet--remove-spaces completion-items)))
-           ;; insert the current result into the completion items list
-           (funcall callback completion-items)))
-       :else
-       (lambda (err)
-         (minuet--handle-chat-completion-timeout
-          context err --response-- #'minuet--claude-get-text-fn "Claude" callback)))
-     minuet--current-requests)))
+  (let* ((options minuet-claude-options)
+         (end-point (plist-get options :end-point))
+         (headers `(("Content-Type" . "application/json")
+                    ("Accept" . "application/json")
+                    ("x-api-key" . ,(minuet--get-api-key (plist-get options :api-key)))
+                    ("anthropic-version" . "2023-06-01")))
+         (body `(,@(plist-get options :optional)
+                 :stream t
+                 :model ,(plist-get options :model)
+                 :system ,(minuet--make-system-prompt (plist-get options :system))
+                 :max_tokens ,(plist-get options :max_tokens)
+                 :messages ,(vconcat
+                             (minuet--eval-value (plist-get options :fewshots))
+                             (--> (minuet--make-chat-llm-shot context options)
+                                  minuet--create-chat-messages-from-list))))
+         (transformed (minuet--apply-request-transform options end-point headers body))
+         (end-point (plist-get transformed :end-point))
+         (headers (plist-get transformed :headers))
+         (body-json (json-serialize (plist-get transformed :body))))
+    (minuet--with-temp-response
+      (push
+       (plz 'post end-point
+         :headers headers
+         :timeout minuet-request-timeout
+         :body body-json
+         :as 'string
+         :filter (minuet--make-process-stream-filter --response--)
+         :then
+         (lambda (json)
+           (when-let* ((result (minuet--stream-decode json #'minuet--claude-get-text-fn))
+                       (completion-items (minuet--parse-completion-itmes-default result))
+                       (completion-items (minuet--filter-context-sequence-in-items
+                                          completion-items
+                                          context))
+                       (completion-items (minuet--remove-spaces completion-items)))
+             ;; insert the current result into the completion items list
+             (funcall callback completion-items)))
+         :else
+         (lambda (err)
+           (minuet--handle-chat-completion-timeout
+            context err --response-- #'minuet--claude-get-text-fn "Claude" callback)))
+       minuet--current-requests))))
 
 (defun minuet--gemini-get-text-fn (json)
   "Function to get the completion from a JSON object for gemini."
@@ -1278,58 +1294,62 @@ to be called when completion items arrive."
 CHAT is a list of plists with :role and :content keys"
   (let (new-chat)
     (dolist (message chat)
-      (let ((gemini-message
-             (pcase (plist-get message :role)
-               ("user"
-                `(:role "user" :parts [(:text ,(plist-get message :content))]))
-               ("assistant"
-                `(:role "model" :parts [(:text ,(plist-get message :content))]))
-               (_ nil))))
-        (when gemini-message
-          (push gemini-message new-chat))))
+      (when-let* ((gemini-message
+                   (pcase (plist-get message :role)
+                     ("user"
+                      `(:role "user" :parts [(:text ,(plist-get message :content))]))
+                     ("assistant"
+                      `(:role "model" :parts [(:text ,(plist-get message :content))])))))
+        (push gemini-message new-chat)))
     (nreverse new-chat)))
 
 (defun minuet--gemini-complete (context callback)
   "Complete code with gemini.
 CONTEXT is to be used to build the prompt.  CALLBACK is the function
 to be called when completion items arrive."
-  (minuet--with-temp-response
-    (push
-     (plz 'post (format "%s/%s:streamGenerateContent?alt=sse"
-                        (plist-get minuet-gemini-options :end-point)
-                        (plist-get minuet-gemini-options :model))
-       :headers `(("Content-Type" . "application/json")
-                  ("x-goog-api-key" . ,(minuet--get-api-key (plist-get minuet-gemini-options :api-key)))
-                  ("Accept" . "application/json"))
-       :timeout minuet-request-timeout
-       :body
-       (json-serialize
-        (let* ((options minuet-gemini-options)
-               (fewshots (minuet--eval-value (plist-get options :fewshots)))
-               (fewshots (minuet--transform-openai-chat-to-gemini-chat fewshots)))
-          `(,@(plist-get options :optional)
-            :system_instruction (:parts (:text ,(minuet--make-system-prompt (plist-get options :system))))
-            :contents ,(vconcat
-                        fewshots
-                        (--> (minuet--make-chat-llm-shot context options)
-                             minuet--create-chat-messages-from-list
-                             minuet--transform-openai-chat-to-gemini-chat)))))
-       :as 'string
-       :filter (minuet--make-process-stream-filter --response--)
-       :then
-       (lambda (json)
-         (when-let* ((result (minuet--stream-decode json #'minuet--gemini-get-text-fn))
-                     (completion-items (minuet--parse-completion-itmes-default result))
-                     (completion-items (minuet--filter-context-sequence-in-items
-                                        completion-items
-                                        context))
-                     (completion-items (minuet--remove-spaces completion-items)))
-           (funcall callback completion-items)))
-       :else
-       (lambda (err)
-         (minuet--handle-chat-completion-timeout
-          context err --response-- #'minuet--gemini-get-text-fn "Gemini" callback)))
-     minuet--current-requests)))
+  (let* ((options minuet-gemini-options)
+         (end-point (format "%s/%s:streamGenerateContent?alt=sse"
+                            (plist-get options :end-point)
+                            (plist-get options :model)))
+         (headers `(("Content-Type" . "application/json")
+                    ("x-goog-api-key" . ,(minuet--get-api-key (plist-get options :api-key)))
+                    ("Accept" . "application/json")))
+         (fewshots (minuet--eval-value (plist-get options :fewshots)))
+         (fewshots (minuet--transform-openai-chat-to-gemini-chat fewshots))
+         (body `(,@(plist-get options :optional)
+                 :system_instruction
+                 (:parts (:text ,(minuet--make-system-prompt (plist-get options :system))))
+                 :contents ,(vconcat
+                             fewshots
+                             (--> (minuet--make-chat-llm-shot context options)
+                                  minuet--create-chat-messages-from-list
+                                  minuet--transform-openai-chat-to-gemini-chat))))
+         (transformed (minuet--apply-request-transform options end-point headers body))
+         (end-point (plist-get transformed :end-point))
+         (headers (plist-get transformed :headers))
+         (body-json (json-serialize (plist-get transformed :body))))
+    (minuet--with-temp-response
+      (push
+       (plz 'post end-point
+         :headers headers
+         :timeout minuet-request-timeout
+         :body body-json
+         :as 'string
+         :filter (minuet--make-process-stream-filter --response--)
+         :then
+         (lambda (json)
+           (when-let* ((result (minuet--stream-decode json #'minuet--gemini-get-text-fn))
+                       (completion-items (minuet--parse-completion-itmes-default result))
+                       (completion-items (minuet--filter-context-sequence-in-items
+                                          completion-items
+                                          context))
+                       (completion-items (minuet--remove-spaces completion-items)))
+             (funcall callback completion-items)))
+         :else
+         (lambda (err)
+           (minuet--handle-chat-completion-timeout
+            context err --response-- #'minuet--gemini-get-text-fn "Gemini" callback)))
+       minuet--current-requests))))
 
 
 (defun minuet--setup-auto-suggestion ()
